@@ -86,6 +86,9 @@ extern "C" {
  * @VALUE_TYPE_PHOTO_URI: internal
  * @VALUE_TYPE_MEDIA_TYPE: internal
  * @VALUE_TYPE_NONE: internal
+ * @VALUE_TYPE_TAG_STRING: internal
+ * @VALUE_TYPE_COLLECTION_ID: internal
+ * @VALUE_TYPE_ICON_PHOTOS: internal 
  * @VALUE_TYPE_LAST: internal offset to last in enum list
  * 
  * Field data types
@@ -103,7 +106,10 @@ typedef enum {
   VALUE_TYPE_URI,
   VALUE_TYPE_PERSON_ID, /* internal */
   VALUE_TYPE_MEDIA_TYPE, /* internal */
-  VALUE_TYPE_LAST = VALUE_TYPE_MEDIA_TYPE
+  VALUE_TYPE_TAG_STRING, /* internal */
+  VALUE_TYPE_COLLECTION_ID, /* internal */
+  VALUE_TYPE_ICON_PHOTOS, /* internal */
+  VALUE_TYPE_LAST = VALUE_TYPE_ICON_PHOTOS
 } flickcurl_field_value_type;
   
 
@@ -163,6 +169,11 @@ typedef enum {
  * @PHOTO_FIELD_usage_candownload: can download
  * @PHOTO_FIELD_usage_canblog: can blog
  * @PHOTO_FIELD_usage_canprint: can print
+ * @PHOTO_FIELD_owner_iconserver: server of owner's icon
+ * @PHOTO_FIELD_owner_iconfarm: farm of owner's icon
+ * @PHOTO_FIELD_original_width: original photo width
+ * @PHOTO_FIELD_original_height: original photo height
+ * @PHOTO_FIELD_views: number of photo views
  * @PHOTO_FIELD_none: internal
  * @PHOTO_FIELD_FIRST: internal offset to first in enum list
  * @PHOTO_FIELD_LAST: internal offset to last in enum list
@@ -225,8 +236,13 @@ typedef enum {
   PHOTO_FIELD_usage_candownload,
   PHOTO_FIELD_usage_canblog,
   PHOTO_FIELD_usage_canprint,
+  PHOTO_FIELD_owner_iconserver,
+  PHOTO_FIELD_owner_iconfarm,
+  PHOTO_FIELD_original_width,
+  PHOTO_FIELD_original_height,
+  PHOTO_FIELD_views,
   PHOTO_FIELD_FIRST = PHOTO_FIELD_dateuploaded,
-  PHOTO_FIELD_LAST = PHOTO_FIELD_usage_canprint
+  PHOTO_FIELD_LAST = PHOTO_FIELD_views
 } flickcurl_photo_field_type;
 
 
@@ -560,6 +576,7 @@ typedef struct {
  * @shapefile_urls: DEPRECATED for @shape->file_urls: NULL-terminated array of pointers to shapefile URLs when present (or NULL)
  * @shapefile_urls_count: DEPRECATED for @shape->file_urls_count: number of entries in @shapefile_urls array
  * @shape: shapefile data (inline data and shapefile urls)
+ * @timezone: timezone of location in 'zoneinfo' format such as “Europe/Paris”.
  *
  * A Place.
  *
@@ -583,6 +600,7 @@ typedef struct {
   int shapefile_urls_count;
 
   struct flickcurl_shapedata_s* shape;
+  char* timezone;
 } flickcurl_place;
   
 
@@ -596,6 +614,8 @@ typedef struct {
  * @data_length: size of @shapedate string
  * @file_urls: NULL-terminated array of pointers to shapefile URLs when present (or NULL)
  * @file_urls_count: number of entries in @shapefile_urls array
+ * @is_donuthole: non-0 if shape IS a donut (a hole)
+ * @has_donuthole: non-0 if shape HAS a donut inside it and it is worth calling places.getShapeHistory on it / flickcurl_places_getShapeHistory()
  *
  * Shape data for a place.
  *
@@ -609,6 +629,8 @@ typedef struct flickcurl_shapedata_s {
   size_t data_length;
   char** file_urls;
   int file_urls_count;
+  int is_donuthole;
+  int has_donuthole;
 } flickcurl_shapedata;
 
 
@@ -900,6 +922,19 @@ typedef struct {
 } flickcurl_blog;
 
 
+/**
+ * flickcurl_blog_service:
+ * @id: ID
+ * @name: Service Name
+ *
+ * A blog service.
+ */
+typedef struct {
+  char* id;
+  char* name;
+} flickcurl_blog_service;
+
+
 /**
  * flickcurl_category:
  * @id: category ID
@@ -922,6 +957,74 @@ struct flickcurl_category_s {
 typedef struct flickcurl_category_s flickcurl_category;
 
 
+/**
+ * flickcurl_photoset:
+ * @id: photoset ID
+ * @primary: primary photo ID
+ * @secret: secret
+ * @server: server
+ * @farm: farm
+ * @photos_count: count of photos in set
+ * @title: title of photoset
+ * @description: description of photoset (may be NULL)
+ * @photos: photos in a photoset (may be NULL)
+ *
+ * A photoset.
+ *
+ */
+typedef struct {
+  char *id;
+  char *primary;
+  char *secret;
+  int server;
+  int farm;
+  int photos_count;
+  char* title;
+  char *description;
+  struct flickcurl_photo_s** photos;
+} flickcurl_photoset;
+
+
+/**
+ * flickcurl_collection:
+ * @id: ID
+ * @child_count: children??
+ * @date_created: date created
+ * @iconlarge: large icon url
+ * @iconsmall: small icon url
+ * @server: server ID
+ * @secret: secret
+ * @title: title
+ * @description: description
+ * @photos: icon photos
+ * @photos_count: number of icon photos
+ * @collections: sub-collections
+ * @collections_count: number of sub-collections
+ * @sets: photo sets
+ * @sets_count: number of photo sets
+ *
+ * A photo collection.  May contain collections OR sets but not both.
+ */
+struct flickcurl_collection_s {
+  char* id;
+  int child_count;
+  int date_created;
+  char *iconlarge;
+  char *iconsmall;
+  int server;
+  char *secret;
+  char *title;
+  char *description;
+  flickcurl_photo **photos;
+  int photos_count;
+  struct flickcurl_collection_s** collections;
+  int collections_count;
+  struct flickcurl_photoset_s** sets;
+  int sets_count;
+};
+typedef struct flickcurl_collection_s flickcurl_collection;
+
+
 /**
  * flickcurl_person_field_type:
  * @PERSON_FIELD_isadmin: is admin field boolean
@@ -1095,8 +1198,8 @@ typedef struct {
  * @tags: A comma-delimited list of tags (or NULL)
  * @tag_mode: Either 'any' for an OR combination of tags, or 'all' for an AND combination. Defaults to 'any' if not specified (or NULL)
  * @text: Free text search (or NULL)
- * @min_upload_date: Minimum upload date as a unix timestamp (or NULL)
- * @max_upload_date: Maximum upload date as a unix timestamp (or NULL)
+ * @min_upload_date: Minimum upload date as a unix timestamp (or 0)
+ * @max_upload_date: Maximum upload date as a unix timestamp (or 0)
  * @min_taken_date: Minimum taken date in the form of a mysql datetime (or NULL)
  * @max_taken_date: Maximum taken date in the form of a mysql datetime (or NULL)
  * @license: Comma-separated list of photo licenses (or NULL)
@@ -1104,14 +1207,14 @@ typedef struct {
  * @privacy_filter: Return photos only matching a certain privacy level.
  * @bbox: A comma-delimited list of 4 values defining the Bounding Box of the area that will be searched.
  * @accuracy: Recorded accuracy level of the location information.  Current range is 1-16 
- * @safe_search: Safe search setting: 1 safe, 2 moderate, 3 restricted.
- * @content_type: Content Type setting: 1 for photos only, 2 for screenshots only, 3 for 'other' only, 4 for all types. (or NULL)
- * @machine_tags: Machine tag search syntax 
+ * @safe_search: Safe search setting: 1 safe, 2 moderate, 3 restricted (or 0).
+ * @content_type: Content Type setting: 1 for photos only, 2 for screenshots only, 3 for 'other' only, 4 for all types. (or 0)
+ * @machine_tags: Machine tag search syntax
  * @machine_tag_mode: Either 'any' for an OR combination of tags, or 'all' for an AND combination. Defaults to 'any' if not specified.
  * @group_id: The id of a group who's pool to search.  If specified, only matching photos posted to the group's pool will be returned. (or NULL)
- * @extras: A comma-delimited list of extra information to fetch for each returned record. Currently supported fields are: <code>license</code>, <code>date_upload</code>, <code>date_taken</code>, <code>owner_name</code>, <code>icon_server</code>, <code>original_format</code>, <code>last_update</code>, <code>geo</code>, <code>tags</code>, <code>machine_tags</code>. (or NULL)
- * @per_page: Number of photos to return per page. If this argument is omitted, it defaults to 100. The maximum allowed value is 500. (or NULL)
- * @page: The page of results to return. If this argument is omitted, it defaults to 1. (or NULL)
+ * @extras: A comma-delimited list of extra information to fetch for each returned record. Currently supported fields are: <code>license</code>, <code>date_upload</code>, <code>date_taken</code>, <code>owner_name</code>, <code>icon_server</code>, <code>original_format</code>, <code>last_update</code>, <code>geo</code>, <code>tags</code>, <code>machine_tags</code>, <code>o_dims</code>, <code>views</code>, <code>media</code>, <code>path_alias</code>, <code>url_sq</code>, <code>url_t</code>, <code>url_s</code>, <code>url_m</code>, <code>url_o</code>. (or NULL)
+ * @per_page: Number of photos to return per page. If this argument is omitted, it defaults to 100. The maximum allowed value is 500. (or 0)
+ * @page: The page of results to return. If this argument is omitted, it defaults to 1. (or 0)
  * @place_id: A Flickr place id. (only used if bbox argument isn't present). Experimental.  Geo queries require some sort of limiting agent in order to prevent the database from crying. This is basically like the check against "parameterless searches" for queries without a geo component.   A tag, for instance, is considered a limiting agent as are user defined min_date_taken and min_date_upload parameters - If no limiting factor is passed we return only photos added in the last 12 hours (though we may extend the limit in the future) (or NULL)
  * @media: "photos" or "videos" (or NULL)
  * @has_geo: non-0 if a photo has been geotagged (or 0)
@@ -1120,6 +1223,10 @@ typedef struct {
  * @radius: A valid radius used for geo queries, greater than zero and less than 20 miles (or 32 kilometers), for use with point-based geo queries. The default value is 5 (km) (or 0.0 for not used)
  * @radius_units: The unit of measure when doing radial geo queries. Valid options are "mi" (miles) and "km" (kilometers). The default is "km" (or NULL)
  * @contacts: (Experimental) Requires @user_id field be set and limits queries to photos beloing to that user's photos.  Valid arguments are 'all' or 'ff' for just friends and family.
+ * @woe_id: A 32-bit identifier that uniquely represents spatial entities. (not used if bbox argument is present).  Same restrictions as @place_id (or 0)
+ * @geo_context: A numeric value representing the photo's geotagginess beyond latitude and longitude. The current list of context IDs is 0: not defined, 1: indoors and 2: outdoors.  Geo queries require some sort of limiting agent in order to prevent the database from crying (or 0)
+ * @is_commons: Limit the scope of the search to only photos that are part of the Flickr Commons project. Default is false (or 0)
+ * @in_gallery: Limit the scope of the search to only photos that are in a gallery. Default is false, search all photos.
  *
  * Search parameters for flickcurl_photos_search()
  */
@@ -1153,35 +1260,13 @@ typedef struct {
   double radius;
   char* radius_units;
   char* contacts;
+  int woe_id;
+  int geo_context;
+  int is_commons;
+  int in_gallery;
 } flickcurl_search_params;
   
 
-/**
- * flickcurl_photoset:
- * @id: photoset ID
- * @primary: primary photo ID
- * @secret: secret
- * @server: server
- * @farm: farm
- * @photos_count: count of photos in set
- * @title: title of photoset
- * @description: description of photoset (may be NULL)
- *
- * A photoset.
- *
- */
-typedef struct {
-  char *id;
-  char *primary;
-  char *secret;
-  int server;
-  int farm;
-  int photos_count;
-  char* title;
-  char *description;
-} flickcurl_photoset;
-
-
 /**
  * flickcurl_size:
  * @label: label
@@ -1422,6 +1507,8 @@ FLICKCURL_API
 void flickcurl_set_write(flickcurl *fc, int is_write);
 FLICKCURL_API
 void flickcurl_set_xml_data(flickcurl *fc, xmlDocPtr doc);
+FLICKCURL_API
+int flickcurl_get_current_request_wait(flickcurl *fc);
 
 /* flickcurl* object set methods */
 FLICKCURL_API
@@ -1433,6 +1520,10 @@ const char* flickcurl_get_auth_token(flickcurl *fc);
 
 /* other flickcurl class destructors */
 FLICKCURL_API
+void flickcurl_free_collection(flickcurl_collection *collection);
+FLICKCURL_API
+void flickcurl_free_collections(flickcurl_collection** collections);
+FLICKCURL_API
 void flickcurl_free_tag_namespace(flickcurl_tag_namespace *tag_nspace);
 FLICKCURL_API
 void flickcurl_free_tag_namespaces(flickcurl_tag_namespace** tag_nspaces);
@@ -1498,6 +1589,24 @@ void flickcurl_free_tag_predicate_values(flickcurl_tag_predicate_value **tag_pvs
 /* get an image URL for a photo in some size */
 FLICKCURL_API
 char* flickcurl_photo_as_source_uri(flickcurl_photo *photo, const char c);
+/* get a photo ID from an image URL */
+FLICKCURL_API
+char* flickcurl_source_uri_as_photo_id(const char *uri);
+/* get a page URL for a photo */
+FLICKCURL_API
+char* flickcurl_photo_as_page_uri(flickcurl_photo *photo);
+/* get a owner icon URL for a photo */
+FLICKCURL_API
+char* flickcurl_user_icon_uri(int farm, int server, char *nsid);
+FLICKCURL_API
+char* flickcurl_photo_as_user_icon_uri(flickcurl_photo *photo);
+/* get a short URL for a photo ID - http://flic.kr */
+FLICKCURL_API
+char* flickcurl_photo_id_as_short_uri(char *photo_id);
+/* get a short URL for a photo object - http://flic.kr */
+FLICKCURL_API
+char* flickcurl_photo_as_short_uri(flickcurl_photo *photo);
+
 /* get labels for various field/types */
 FLICKCURL_API
 const char* flickcurl_get_photo_field_label(flickcurl_photo_field_type field);
@@ -1567,6 +1676,17 @@ FLICKCURL_API
 int flickcurl_blogs_postPhoto(flickcurl* fc, const char* blog_id, const char* photo_id, const char* title, const char* description, const char* blog_password);
 FLICKCURL_API
 void flickcurl_free_blogs(flickcurl_blog **blogs_object);
+FLICKCURL_API
+flickcurl_blog_service** flickcurl_blogs_getServices(flickcurl* fc);
+FLICKCURL_API
+void flickcurl_free_blog_services(flickcurl_blog_service **blog_services_object);
+
+
+/* flickr.collections */
+FLICKCURL_API
+flickcurl_collection* flickcurl_collections_getInfo(flickcurl* fc, const char* collection_id);
+FLICKCURL_API
+flickcurl_collection* flickcurl_collections_getTree(flickcurl* fc, const char* collection_id, const char* user_id);
 
 /* flickr.commons */
 FLICKCURL_API
@@ -1641,6 +1761,8 @@ FLICKCURL_API
 flickcurl_tag_predicate_value** flickcurl_machinetags_getPredicates(flickcurl* fc, const char *nspace, int per_page, int page);
 FLICKCURL_API
 flickcurl_tag_predicate_value** flickcurl_machinetags_getValues(flickcurl* fc, const char *nspace, const char* predicate, int per_page, int page);
+FLICKCURL_API
+flickcurl_tag_predicate_value** flickcurl_machinetags_getRecentValues(flickcurl* fc, const char *nspace, const char* predicate, int added_since);
 
 /* flickr.panda */
 FLICKCURL_API
@@ -1756,6 +1878,8 @@ flickcurl_place* flickcurl_places_getInfo(flickcurl* fc, const char* place_id, c
 FLICKCURL_API
 flickcurl_place* flickcurl_places_getInfo2(flickcurl* fc, const char* place_id, const int woe_id);
 FLICKCURL_API
+flickcurl_place** flickcurl_places_getTopPlacesList(flickcurl* fc, flickcurl_place_type place_type, const char* date, int woe_id, const char* place_id);
+FLICKCURL_API
 flickcurl_place* flickcurl_places_getInfoByUrl(flickcurl* fc, const char* url);
 FLICKCURL_API
 flickcurl_place* flickcurl_places_resolvePlaceId(flickcurl* fc, const char* place_id);
@@ -1808,6 +1932,8 @@ FLICKCURL_API
 int flickcurl_photos_comments_editComment(flickcurl* fc, const char* comment_id, const char* comment_text);
 FLICKCURL_API
 flickcurl_comment** flickcurl_photos_comments_getList(flickcurl* fc, const char* photo_id);
+FLICKCURL_API
+flickcurl_photos_list* flickcurl_photos_comments_getRecentForContacts_params(flickcurl* fc, int date_lastcomment, const char* contacts_filter, flickcurl_photos_list_params* list_params);
 
 /* flickr.photos.geo */
 FLICKCURL_API
