@@ -64,11 +64,11 @@ flickcurl_build_tags(flickcurl* fc, flickcurl_photo* photo,
                      xmlXPathContextPtr xpathCtx, const xmlChar* xpathExpr,
                      int* tag_count_p)
 {
-  flickcurl_tag** tags=NULL;
+  flickcurl_tag** tags = NULL;
   int nodes_count;
   int tag_count;
   int i;
-  xmlXPathObjectPtr xpathObj=NULL;
+  xmlXPathObjectPtr xpathObj = NULL;
   xmlNodeSetPtr nodes;
   
   /* Now do tags */
@@ -76,76 +76,76 @@ flickcurl_build_tags(flickcurl* fc, flickcurl_photo* photo,
   if(!xpathObj) {
     flickcurl_error(fc, "Unable to evaluate XPath expression \"%s\"", 
                     xpathExpr);
-    fc->failed=1;
+    fc->failed = 1;
     goto tidy;
   }
   
-  nodes=xpathObj->nodesetval;
+  nodes = xpathObj->nodesetval;
   /* This is a max size - it can include nodes that are CDATA */
-  nodes_count=xmlXPathNodeSetGetLength(nodes);
-  tags=(flickcurl_tag**)calloc(sizeof(flickcurl_tag*), nodes_count+1);
+  nodes_count = xmlXPathNodeSetGetLength(nodes);
+  tags = (flickcurl_tag**)calloc(sizeof(flickcurl_tag*), nodes_count+1);
   
-  for(i=0, tag_count=0; i < nodes_count; i++) {
-    xmlNodePtr node=nodes->nodeTab[i];
+  for(i = 0, tag_count = 0; i < nodes_count; i++) {
+    xmlNodePtr node = nodes->nodeTab[i];
     xmlAttr* attr;
     flickcurl_tag* t;
-    int saw_clean=0;
+    int saw_clean = 0;
     xmlNodePtr chnode;
     
     if(node->type != XML_ELEMENT_NODE) {
       flickcurl_error(fc, "Got unexpected node type %d", node->type);
-      fc->failed=1;
+      fc->failed = 1;
       break;
     }
     
-    t=(flickcurl_tag*)calloc(sizeof(flickcurl_tag), 1);
-    t->photo=photo;
+    t = (flickcurl_tag*)calloc(sizeof(flickcurl_tag), 1);
+    t->photo = photo;
     
-    for(attr=node->properties; attr; attr=attr->next) {
-      const char *attr_name=(const char*)attr->name;
+    for(attr = node->properties; attr; attr = attr->next) {
+      const char *attr_name = (const char*)attr->name;
       char *attr_value;
 
-      attr_value=(char*)malloc(strlen((const char*)attr->children->content)+1);
+      attr_value = (char*)malloc(strlen((const char*)attr->children->content)+1);
       strcpy(attr_value, (const char*)attr->children->content);
       
       if(!strcmp(attr_name, "id"))
-        t->id=attr_value;
+        t->id = attr_value;
       else if(!strcmp(attr_name, "author"))
-        t->author=attr_value;
+        t->author = attr_value;
       else if(!strcmp(attr_name, "authorname"))
-        t->authorname=attr_value;
+        t->authorname = attr_value;
       else if(!strcmp(attr_name, "raw"))
-        t->raw=attr_value;
+        t->raw = attr_value;
       else if(!strcmp(attr_name, "clean")) {
-        t->cooked=attr_value;
+        t->cooked = attr_value;
         /* If we see @clean we are expecting
-         * <tag clean="cooked"><raw>raw</raw></tag>
+         * <tag clean = "cooked"><raw>raw</raw></tag>
          */
-        saw_clean=1;
+        saw_clean = 1;
       } else if(!strcmp(attr_name, "machine_tag")) {
-        t->machine_tag=atoi(attr_value);
+        t->machine_tag = atoi(attr_value);
         free(attr_value);
       } else if(!strcmp(attr_name, "count")) {
-        t->count=atoi(attr_value);
+        t->count = atoi(attr_value);
         free(attr_value);
       } else if(!strcmp(attr_name, "score")) {
-        /* from tags.getHotList <tag score="NN">TAG</tag> */
-        t->count=atoi(attr_value);
+        /* from tags.getHotList <tag score = "NN">TAG</tag> */
+        t->count = atoi(attr_value);
         free(attr_value);
       }
     }
 
     /* Walk children nodes for <raw> element or text */
-    for(chnode=node->children; chnode; chnode=chnode->next) {
-      const char *chnode_name=(const char*)chnode->name;
+    for(chnode = node->children; chnode; chnode = chnode->next) {
+      const char *chnode_name = (const char*)chnode->name;
       if(chnode->type == XML_ELEMENT_NODE) {
         if(saw_clean && !strcmp(chnode_name, "raw")) {
-          t->raw=(char*)malloc(strlen((const char*)chnode->children->content)+1);
+          t->raw = (char*)malloc(strlen((const char*)chnode->children->content)+1);
           strcpy(t->raw, (const char*)chnode->children->content);
         }
       } else if(chnode->type == XML_TEXT_NODE) {
         if(!saw_clean) {
-          t->cooked=(char*)malloc(strlen((const char*)chnode->content)+1);
+          t->cooked = (char*)malloc(strlen((const char*)chnode->content)+1);
           strcpy(t->cooked, (const char*)chnode->content);
         }
       }
@@ -159,11 +159,11 @@ flickcurl_build_tags(flickcurl* fc, flickcurl_photo* photo,
     if(fc->tag_handler)
       fc->tag_handler(fc->tag_data, t);
     
-    tags[tag_count++]=t;
+    tags[tag_count++] = t;
   } /* for nodes */
 
   if(tag_count_p)
-    *tag_count_p=tag_count;
+    *tag_count_p = tag_count;
   
  tidy:
   if(xpathObj)
@@ -173,6 +173,56 @@ flickcurl_build_tags(flickcurl* fc, flickcurl_photo* photo,
 }
 
 
+flickcurl_tag**
+flickcurl_build_tags_from_string(flickcurl* fc, flickcurl_photo* photo,
+                                 const char *string, int *tag_count_p)
+{
+  flickcurl_tag** tags = NULL;
+  int nodes_count;
+  int tag_count;
+  int i;
+  
+  nodes_count = 0;
+  for(i = 0; string[i]; i++) {
+    if(string[i] == ' ')
+      nodes_count++;
+  }
+  
+  tags = (flickcurl_tag**)calloc(sizeof(flickcurl_tag*), nodes_count+1);
+  
+  for(i = 0, tag_count = 0; i < nodes_count; i++) {
+    flickcurl_tag* t;
+    const char *p = string;
+    size_t len;
+    
+    t = (flickcurl_tag*)calloc(sizeof(flickcurl_tag), 1);
+    t->photo = photo;
+
+    while(*p && *p != ' ')
+      p++;
+    
+    len = p-string;
+
+    t->cooked = (char*)malloc(len+1);
+    strncpy(t->cooked, string, len);
+    t->cooked[len] = '\0';
+    
+    if(fc->tag_handler)
+      fc->tag_handler(fc->tag_data, t);
+    
+    tags[tag_count++] = t;
+
+    /* move past space */
+    string = p+1;
+  }
+
+  if(tag_count_p)
+    *tag_count_p = tag_count;
+  
+  return tags;
+}
+
+
 /*
  * flickcurl_free_tag_cluster:
  * @tc: tag cluster object
@@ -186,7 +236,7 @@ flickcurl_free_tag_cluster(flickcurl_tag_cluster *tc)
 
   if(tc->tags) {
     int i;
-    for(i=0; tc->tags[i]; i++)
+    for(i = 0; tc->tags[i]; i++)
       free(tc->tags[i]);
     free(tc->tags);
   }
@@ -207,7 +257,7 @@ flickcurl_free_tag_clusters(flickcurl_tag_clusters *tcs)
 
   if(tcs->clusters) {
     int i;
-    for(i=0; tcs->clusters[i]; i++)
+    for(i = 0; tcs->clusters[i]; i++)
       flickcurl_free_tag_cluster(tcs->clusters[i]);
     free(tcs->clusters);
   }
@@ -220,71 +270,71 @@ flickcurl_build_tag_clusters(flickcurl* fc,
                              xmlXPathContextPtr xpathCtx,
                              const xmlChar* xpathExpr)
 {
-  flickcurl_tag_clusters* tcs=NULL;
+  flickcurl_tag_clusters* tcs = NULL;
   int nodes_count;
   int cluster_count;
   int i;
-  xmlXPathObjectPtr xpathObj=NULL;
+  xmlXPathObjectPtr xpathObj = NULL;
   xmlNodeSetPtr nodes;
   
   xpathObj = xmlXPathEvalExpression(xpathExpr, xpathCtx);
   if(!xpathObj) {
     flickcurl_error(fc, "Unable to evaluate XPath expression \"%s\"", 
                     xpathExpr);
-    fc->failed=1;
+    fc->failed = 1;
     goto tidy;
   }
   
-  tcs=(flickcurl_tag_clusters*)calloc(sizeof(flickcurl_tag_clusters), 1);
+  tcs = (flickcurl_tag_clusters*)calloc(sizeof(flickcurl_tag_clusters), 1);
 
   /* <cluster> XML element nodes */
-  nodes=xpathObj->nodesetval;
+  nodes = xpathObj->nodesetval;
   /* This is a max size - it can include nodes that are CDATA */
-  nodes_count=xmlXPathNodeSetGetLength(nodes);
+  nodes_count = xmlXPathNodeSetGetLength(nodes);
 
-  tcs->clusters=(flickcurl_tag_cluster**)calloc(sizeof(flickcurl_tag_cluster*), nodes_count+1);
+  tcs->clusters = (flickcurl_tag_cluster**)calloc(sizeof(flickcurl_tag_cluster*), nodes_count+1);
   
-  for(i=0, cluster_count=0; i < nodes_count; i++) {
-    xmlNodePtr node=nodes->nodeTab[i];
+  for(i = 0, cluster_count = 0; i < nodes_count; i++) {
+    xmlNodePtr node = nodes->nodeTab[i];
     xmlAttr* attr;
     xmlNodePtr chnode;
-    flickcurl_tag_cluster* tc=NULL;
+    flickcurl_tag_cluster* tc = NULL;
     int tags_count= -1;
     
     if(node->type != XML_ELEMENT_NODE) {
       flickcurl_error(fc, "Got unexpected node type %d", node->type);
-      fc->failed=1;
+      fc->failed = 1;
       break;
     }
 
-    tc=(flickcurl_tag_cluster*)calloc(sizeof(flickcurl_tag_cluster), 1);
+    tc = (flickcurl_tag_cluster*)calloc(sizeof(flickcurl_tag_cluster), 1);
     if(!tc) {
-      fc->failed=1;
+      fc->failed = 1;
       break;
     }
 
     /* get <cluster @total> */
-    for(attr=node->properties; attr; attr=attr->next) {
-      const char *attr_name=(const char*)attr->name;
-      const char *attr_value=(const char*)attr->children->content;
+    for(attr = node->properties; attr; attr = attr->next) {
+      const char *attr_name = (const char*)attr->name;
+      const char *attr_value = (const char*)attr->children->content;
       
       if(!strcmp(attr_name, "total"))
-        tags_count=atoi(attr_value);
+        tags_count = atoi(attr_value);
     }
 
-    if(tags_count <=0)
+    if(tags_count <= 0)
       continue;
     
-    tc->tags=(char**)calloc(sizeof(char*), tags_count+1);
+    tc->tags = (char**)calloc(sizeof(char*), tags_count+1);
 
     /* Walk children nodes of <cluster> for <tag> elements */
-    for(chnode=node->children; chnode; chnode=chnode->next) {
-      const char *chnode_name=(const char*)chnode->name;
+    for(chnode = node->children; chnode; chnode = chnode->next) {
+      const char *chnode_name = (const char*)chnode->name;
       if(chnode->type == XML_ELEMENT_NODE && !strcmp(chnode_name, "tag")) {
-        size_t len=strlen((const char*)chnode->children->content);
-        char *tag_name=(char*)malloc(len+1);
+        size_t len = strlen((const char*)chnode->children->content);
+        char *tag_name = (char*)malloc(len+1);
         strcpy(tag_name, (const char*)chnode->children->content);
-        tc->tags[tc->count++]=tag_name;
+        tc->tags[tc->count++] = tag_name;
 
 #if FLICKCURL_DEBUG > 1
         fprintf(stderr, "cluster #%d tag #%d: %s\n", tcs->count, tc->count,
@@ -293,12 +343,12 @@ flickcurl_build_tag_clusters(flickcurl* fc,
     
       }
     }
-    tc->tags[tc->count]=NULL;
+    tc->tags[tc->count] = NULL;
     
-    tcs->clusters[tcs->count++]=tc;
+    tcs->clusters[tcs->count++] = tc;
   } /* for <cluster> nodes */
 
-  tcs->clusters[tcs->count]=NULL;
+  tcs->clusters[tcs->count] = NULL;
 
  tidy:
   if(xpathObj)
